# Copyright(c) 2003, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: runcommand.tcl,v 1.19 2004/01/30 20:28:37 tsmith Exp $
#
# Run a McIDAS command, sending output to the text window.
#
#    RunCommand "[flags] command"
#
#	command	| McIDAS command to be executed
#
#	flags:
#		-noecho  | do NOT echo command in the McIDAS text window
# 		           (even if echoing is turned on in the GUI)
#		-nowatch | do NOT change the cursor to a watch
#		-nohist  | do NOT save the command to MCCMDHIST.TXT
#
#  Example usage:   RunCommand "-noecho -nowatch -nohist EG $frame"
#
#---------------

proc RunCommand { command } \
{
   global env

   # Check for flags in the command string, if one is found
   # remove it from the command string.

   # NOWATCH
   set nowatch_flag [CheckForRunCommandFlag "nowatch" $command]
   if { $nowatch_flag } \
   {
   	set command [RemoveFlagFromCommandString "nowatch" $command]
   }
   
   # NOECHO
   set noecho_flag [CheckForRunCommandFlag "noecho" $command]
   if { $noecho_flag } \
   {
   	set command [RemoveFlagFromCommandString "noecho" $command]
   }   
   
   # NOHIST
   set nohist_flag [CheckForRunCommandFlag "nohist" $command]
   if { $nohist_flag } \
   {
   	set command [RemoveFlagFromCommandString "nohist" $command]
   }
   
   # Set the cursor to a watch unless the -nowatch flag was used.
   if { $nowatch_flag == 0 } { showBusyStatus }

   # If the command echoing is turned on and the -noecho flag was not
   # used then echo the command to the McIDAS text window.
   if { $noecho_flag == 0 && [UCpeek [ucLocation gui_cmd_echo]] } \
   {
	regsub {( |\t)*&( |\t)*$} $command "" cmdstr
	puts -nonewline "\x03\x0a"
	puts $cmdstr
   }   
      
   # Save the command to MCCMDHIST.TXT unless the -nohist flag
   # was used.
   if { $nohist_flag == 0 } { SaveInCommandHistory $command }
   
   # Finally we can run the command
   if { [running_mcwish] } \
   {
	mcidas exec $command
   } \
   else \
   {
	set executable "[string tolower [lindex $command 0]].k"
	catch {eval exec $executable [lrange $command 1 end]}
   }

   # Put cursor back to the default
   if { $nowatch_flag == 0 } { showIdleStatus }
}


###############################################################
# SaveInCommandHistory
###############################################################
proc SaveInCommandHistory { command_string } \
{
   # Save the command in the McIDAS command history text file
   set history_file [GetLocalDataPath "MCCMDHIST.TXT"]
   set max_lines [UCpeek [ucLocation gui_cmd_history]]
   set num_lines 0

   # Open file and read into an array
   if { [file isfile $history_file] } \
   {
	set i 0
	set inStream [open $history_file "r"]
	while { [gets $inStream line] >= 0 } \
	{
	   set file_contents($i) $line
	   incr i
	}
	set num_lines $i
	catch { close $inStream }
   }

   # Add new command to array
   set file_contents($num_lines) $command_string
   incr num_lines

   if { $num_lines >= $max_lines } \
   {
   	set beginning_index [expr $num_lines - $max_lines]
   	if { $beginning_index < 0 } { set beginning_index 0 }
   } \
   else \
   {
	set beginning_index 0
   }

   # Write contents back to history file
   set outStream [open $history_file "w"]
   for { set i $beginning_index } { $i < $num_lines } { incr i } \
   {
	if { $i < [expr $num_lines - 1] } \
	{
	   puts $outStream $file_contents($i)
	} \
	else \
	{
	   puts -nonewline $outStream $file_contents($i)
	}
   }
   catch { close $outStream }
}


###############################################################
# CheckForRunCommandFlag - return 1 if flag exists in command
#                          string otherwise return 0.
###############################################################
proc CheckForRunCommandFlag { flag_type command_string } \
{
   if { [string match "*-$flag_type*" $command_string] } \
   {
	return 1	
   } \
   else \
   {
	return 0
   }
}


###############################################################
# RemoveFlagFromCommandString - return the command string with
#                               the flag removed.
###############################################################
proc RemoveFlagFromCommandString { flag_type command_string } \
{
   set beg_index [string first "-$flag_type" $command_string ]
   set end_index [expr { $beg_index + 7 }]

   if { $::tk_version <= 8.1 } \
   {
	return "[string range $command_string 0 [expr $beg_index-1]] \
		[string range $command_string [expr $end_index+1] end]"
   } \
   else \
   {
	return [string replace $command_string $beg_index $end_index]
   }
}

